# Copyright (C) 2021, KylinSoft Co., Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
import re
import os
import sys

arg_lang = ""

################################
# 常量

# for get_env_lang()
STR_GET_ENV_LANG_ZH     = "语言环境为中文"
STR_GET_ENV_LANG_EN     = "语言环境为英文"
STR_GET_ENV_LANG_UNKNOW = "语言环境未知"

# for is_root()
STR_IS_ROOT_TRUE  = "当前用户为root权限"
STR_IS_ROOT_FALSE = "当前用户没有root权限"

################################
# 环境检查函数

def get_env_lang():
#    lang = os.getenv("LANG")
#    if lang.startswith("zh"):
#        return STR_GET_ENV_LANG_ZH
#    elif lang.startswith("en"):
#        return STR_GET_ENV_LANG_EN
#    else:
#        return STR_GET_ENV_LANG_UNKNOW
#
    if arg_lang == "zh":
        return STR_GET_ENV_LANG_ZH
    elif arg_lang == "en":
        return STR_GET_ENV_LANG_EN
    else:
        return STR_GET_ENV_LANG_UNKNOW

def is_root():
    if os.geteuid() == 0:
        print(STR_IS_ROOT_TRUE)
        return True
    else:
        print(STR_IS_ROOT_FALSE)
        return False

################################
# 辅助函数
def l_print(zh_str, en_str) :
    if STR_GET_ENV_LANG_ZH == get_env_lang() :
        print(zh_str);
    else :
        print(en_str);

################################
# 功能函数

def check_ssh_failed():
    # 定义正则表达式，用于匹配SSH登录失败日志行
    # regex = r'^(\S+).*authentication failure.*rhost=(\S+).*user=(\S+).*'
    regex = r'^(\S+).*Failed password for.* (\S+).*from (\S+).*port (\S+).* ssh2'


    # 定义常量，指定最大失败次数
    MAX_FAILD_NUM = 5

    # 初始化计数器
    counter = {}

    # 打开SSH日志文件并逐行读取
    with open('/var/log/auth.log', 'r') as f:
        for line in f:
            # 使用正则表达式匹配登录失败日志行
            match = re.match(regex, line)
            if match:
                # 如果找到匹配项，提取IP地址和用户名
                ip_address = match.group(2)
                username = match.group(3)

                # 检查是否需要记录该IP地址和用户
                if ip_address+":"+username in counter:
                    # 如果IP地址已存在于计数器中，则增加计数器
                    counter[ip_address+":"+username] += 1
                else:
                    # 如果IP地址不存在于计数器中，则添加新的计数器
                    counter[ip_address+":"+username] = 1

    has_faild = False
    for key, cnt in counter.items():
        if cnt >= MAX_FAILD_NUM:
            has_faild = True
        # 如果失败次数超过最大限制，则记录IP地址和用户
            l_print(f'[WARNING] IP地址：{key.split(":")[1]}，用户：{key.split(":")[0]}, 失败次数：{cnt}',
                    f'[WARNING] IP: {key.split(":")[1]}，USER: {key.split(":")[0]}, Count of Failures：{cnt}')

    if False == has_faild :
        l_print(f'[OK] 没有发现大量登录失败',
                f'[OK] No significant login failures found')


################################
# main
if __name__ == "__main__":
    arg_lang = sys.argv[1]

    if False == is_root() :
        l_print("[WARNING] 非root权限，此程序需要root权限",
                "[WARNING] Non root permission, this program requires root permission")
        exit(1)

    check_ssh_failed()
    exit(0)

